var preload_ongoingReq = {};
function loadUrl(url, onload, onfail, opt) {
    if (preload_ongoingReq[url]) {
        return;
    }
    if (url.substring(0, 4) === 'http') {
        url = 'xiaomi_vip/loadUrl?url=' + url;
    }
    var option = $.extend({
        type: 'get',
        dataType: 'json'
    }, opt);
    option.success = function(res, status, xhr) {
        if ($.isFunction(onload)) {
            onload(res);
        }
    };
    option.error = function(xhr, status, errorThrown) {
        var errMsg = status + ' ' + errorThrown;
        console.log('loadUrl, error on ' + url + ', status = ' + errMsg);
        if (xhr.status == 200 && status === 'parsererror') {
            if ($.isFunction(onload)) {
                onload(xhr.responseText);
            }
        } else if ($.isFunction(onfail)) {
            onfail(xhr, status, errorThrown);
        }
    };
    option.complete = function(xhr, status) {
        preload_ongoingReq[url] = false;
    }
    $.ajax(url, option);
}
function parseParams(funcArgs){
    var args = Array.prototype.slice.call(funcArgs, 0);
    var ret = {methodArgs:null,onload:null,onfail:null};
    if(args.length > 0){
        var callbackArray = [];
        var i = args.length - 1;
        var j = 0;
        for (; i >= 0; i--) {
            if (args[i] && typeof args[i]['call'] == 'function') {
                callbackArray[j] = args[i];j++;
            } else{
                if(typeof args[i] == 'undefined' || typeof args[i] == 'null'){
                    j++;
                }
                else{
                    break;
                }
            }
        }
        if (callbackArray.length == 1) {
            ret.onload = callbackArray[0];
        } else if (callbackArray.length > 1) {
            ret.onfail = callbackArray[0];
            ret.onload = callbackArray[1];
        }
        ret.methodArgs = j > 0 ? args.slice(0, j * -1) : args;
    }
    return ret;
}
function createAPI(url, name) {
    var apiUrl = url;
    var funcName = name;
    return function(){
        var parsed = parseParams(arguments);
        loadUrl(apiUrl + '/' + funcName + (parsed.methodArgs ? '?args=' + JSON.stringify(parsed.methodArgs) : ''),
            parsed.onload, parsed.onfail);
    };
}
function loadVipData() {
    var parsed = parseParams(arguments);
    var type = parsed.methodArgs.splice(0, 1);
    loadUrl('xiaomi_vip/loadData/' + type + '?args=' + JSON.stringify(parsed.methodArgs),
        parsed.onload, parsed.onfail);
}
function vipHttpRequest(url, method, params, onload, onfail){
    var args = 'url=' + url + '&method=' + method + '&args=' + JSON.stringify(params);
    loadUrl('xiaomi_vip/sendHttp' + '?' + args, onload, onfail);
}
function vipLoadCache(key, onload) {
    loadUrl('xiaomi_vip/load_cache?cacheKey=' + key, onload);
}

function loadcss(url, onload, onfail) {
    loadUrl(url, function(res) {
        var style = $('<style type = "text/css">' + res + '</style>');
        $('head').append(style);
        onload(res);
    }, onfail, { dataType: 'text' });
}

function loadjs(url, onload, onfail) {
    loadUrl(url, onload, onfail, { dataType: 'script' });
}

function loadAssets(url, str, re, loadFunc, complete, checker) {
    var ret = [];
    var match;

    if (!checker) {
        checker = function(match) {
            if (match[1].indexOf('jquery', 0) === -1) {
                return match[1];
            }
        };
    }

    while (match = re.exec(str)) {
        var file = checker(match);
        if (file) {
            ret.push(file);
        }
    }
    console.log('loadAssets, ret = ' + JSON.stringify(ret));
    if (!ret.length) {
        return;
    }
    var succCount = 0;
    var failCount = 0;
    function completeOne(succ) {
        if (succ) {
            succCount++;
        } else {
            failCount++;
        }
        if (succCount + failCount == ret.length && complete) {
            complete(succCount, failCount);
        }
    }
    var host = url.substring(0, url.lastIndexOf('/') + 1);
    for (var i = 0; i < ret.length; i++) {
        var assetUrl = ret[i];
        var fileurl = assetUrl.substring(0, 4) === 'http' ? assetUrl : host + assetUrl;
        loadFunc(fileurl, function(res) {
            console.log('success to load ' + fileurl);
            completeOne(true);
        },
        function(status) {
            console.log('failed to load ' + fileurl);
            completeOne();
        });
    }
}

var pagere = /id\s*=\s*['"]([^'"]+)['"]\s*data-role\s*=\s*['"]page['""]/g;
var jsre = /<script\s*(type=['"]text\/javascript['"])?\s*src="([^"]+)"[\s]*(\/>|><\/script>)/g;

function insertContent(res) {
    var bodyStart = res.indexOf('<body>');
    bodyStart = bodyStart ? bodyStart + '<body>'.length : bodyStart;

    var bodyEnd = res.indexOf('</body>');

    if (bodyStart > 0 && bodyEnd > 0) {
        var body = res.substring(bodyStart, bodyEnd);
        body = body.replace(jsre, '').trim();

        $('body').append($(body));

        console.log('insertContent');

        var match = pagere.exec(body);
        if (match) {
            var firstPageId = match[1];
            return firstPageId;
        }
    }
}

var preload_page_retryTimes = 0;

function loadPage(url) {
    console.log('loadPage, url = ' + url);

    loadUrl(url, function(res, isText, type) {
        var queryStart = url.indexOf('?');
        if (queryStart > 0) {
            var queryString = url.substring(queryStart);
            console.log('loadPage, queryString = ' + queryString);
            window.extraSearch = queryString;
        }

        var cssre = /<link rel="stylesheet" href="([^"]+)".*(\/>|><\/link>)/g;
        loadAssets(url, res, cssre, loadcss, function() {
            var firstPageId = insertContent(res);

            loadAssets(url, res, jsre, loadjs,
                function() {
                    console.log('loadPage, firstPageId = ' + firstPageId);
                    if (window.context) {
                        if ($.isFunction(context.initLoadPage)) {
                            context.initLoadPage();
                        } else {
                            console.log('change to page ' + firstPageId);
                            context.changePage(firstPageId, null, { setHash: true });
                        }
                    }
                    loadUrl('xiaomi_vip/onload?url=' + url);
                },
                function(match) {
                    if (match[2].indexOf('jquery', 0) === -1) {
                        return match[2];
                    }
                });
        });
    },
    function(status) {
        console.log('loadPage failed for ' + url);
        if (preload_page_retryTimes < 3) {
            preload_page_retryTimes++;
            setTimeout(loadPage.bind(null, url), 100);
        }
    },
    { dataType: 'text' });
}

function loadFinish() {
    console.log('preload.js loaded, notify XiaomiVip of loading finish');
    loadUrl('xiaomi_vip/initialized?location=' + location.href);
};

loadFinish();
