precision highp float;
uniform sampler2D vTexture;
varying vec2 vTextureCo;
uniform float uTime;
uniform vec2 time_range;// 时间范围
uniform vec4 pos_range;// 位移范围vec4(开始x，开始y，结束x，结束y)
uniform float angle;
uniform float zoom;

//这个 shader 中的 motion blur 和 transform.frag 中的 motion blur 是不一样的
//这里的 blur 为定向模糊，需要外部传入模糊的角度和长度才能工作，然不是依赖时间控制变化
uniform float motion_blur_angle;
uniform float motion_blur_length;
uniform bool motion_blur_enable;

uniform bool interpolator_enable;

const vec2 screenSize = vec2(720., 1280.);
const float PI = 3.14159;

vec2 getUV(vec2 position, vec2 uv, float fstep)
{
    vec2 center = vec2(0.5);//屏幕中间的比例点

    float sx = 1./zoom;
    float t = angle * PI / 180.;//角度转弧度
    vec2 trans = position;

    vec2 futureUv = (uv + trans - center);

    mat3 rot = mat3(sx * cos(t), -sx/1.78 * sin(t), 0.,
    sx * sin(t)*1.78, sx * cos(t), 0.,
    0., 0., 1.0);
    vec3 ruv = rot * vec3(futureUv, 1.0);
    futureUv = center + ruv.xy;//然后再放回去

    return futureUv;
}

vec2 getCurrentPosition(float fstep){
    vec4 current_pos_range = (pos_range - vec4(screenSize/2.0, screenSize/2.0)) / vec4(screenSize, screenSize);//计算出和屏幕的比例 vec4(start_x_f,start_y_f,end_x_f,end_y_f)
    vec2 center = vec2(0.5);//屏幕中间的比例点
    vec2 pos_start = current_pos_range.xy * -1.0;
    vec2 pos_end = current_pos_range.zw * -1.0;
    float tx = pos_start.x + (pos_end.x - pos_start.x) * fstep;//tx:translationX ，fstep 这个时间点下的偏移量
    float ty = -1. *(pos_start.y + (pos_end.y - pos_start.y) * fstep);
    return vec2(tx, ty);
}

vec2 getMotionBlurUv(vec2 position, vec2 uv, float blur_angle, float length)
{
    vec2 center = vec2(0.5);//屏幕中间的比例点

    float sx = 1./zoom;
    float t = angle * PI / 180.;//角度转弧度

    float tx = position.x + cos(blur_angle* PI / 180.) * length / screenSize.x;//tx:translationX ，fstep 这个时间点下的偏移量
    float ty = position.y + sin(blur_angle* PI / 180.) * length / screenSize.y;
    vec2 trans = vec2(tx, ty);

    vec2 futureUv = (uv + trans - center);

    mat3 rot = mat3(sx * cos(t), -sx/1.78 * sin(t), 0.,
    sx * sin(t)*1.78, sx * cos(t), 0.,
    0., 0., 1.0);
    vec3 ruv = rot * vec3(futureUv, 1.0);
    futureUv = center + ruv.xy;//然后再放回去

    return futureUv;
}

void main() {
    vec2 uv = vTextureCo;//texture 原来的 uv

    vec2 splitUv = vec2(uv.xy);

    float time_length = (time_range.y - time_range.x);
    float fstep0 = (uTime-time_range.x) / time_length;//位于整个动画的什么位置
    if (fstep0 > 1.0) fstep0 = 1.0;

    vec2 position = getCurrentPosition(fstep0);

    vec2 muv = getUV(position, splitUv, fstep0);//当前的 uv

    float remapped_angle = motion_blur_angle + 90.;
    if (motion_blur_enable){
        //角度指向的方向和反向的反向各取一半长度
        vec2 tuv = getMotionBlurUv(position, splitUv, remapped_angle, motion_blur_length * 0.5);
        vec2 puv = getMotionBlurUv(position, splitUv, remapped_angle + 180., motion_blur_length * 0.5);

        float step = 25.0;
        int istep = int(step);

        vec4 result = vec4(0.0);

        vec2 dir_step = (tuv.xy - puv) / step;
        for (int i = 0; i < istep; i++) {
            vec2 xuv = puv + dir_step * float(i);
            if ((xuv.x >=0.0) && (xuv.y >= 0.0) && (xuv.x <= 1.0) && (xuv.y <= 1.0)) {
                //将多次采样的结果放到 result 上
                result += texture2D(vTexture, xuv);
            }
        }

        if (result.a/step < 1.0){ //由于透明度不满 step 的话会有黑色，这些通通丢弃，只保留全部画面由 alpha==1 的画面叠加的结果
            discard;
        } else {
            gl_FragColor = vec4(result.xyz / step, 1.0);// 除以 step 取回平均值
        }
    } else {
        if ((muv.x >=0.0) && (muv.y >= 0.0) && (muv.x <= 1.0) && (muv.y <= 1.0)) {
            //不做限制的话周边可能会有条纹产生
            gl_FragColor = texture2D(vTexture, muv);
        }else{
            discard;
        }
    }
}