precision highp float;
uniform sampler2D vTexture;
varying vec2 vTextureCo;
uniform float uTime;
uniform vec2 time_range;// 时间范围
uniform vec4 pos_range;// 位移范围vec4(开始x，开始y，结束x，结束y)
uniform vec2 angle_range;// 角度范围√
uniform vec2 zoom_range;// 放缩范围 vec2(缩放x，缩放y)

uniform bool mirror_fill;
uniform bool interpolator_enable;
uniform bool compact_enable;
uniform bool motion_blur_enable;

const vec2 screenSize = vec2(720., 1280.);
const float PI = 3.14159;

float bezierInterpolate(float t){
    float dt = 1.0-t;
    return 0.66*dt*dt*dt + 1.64*3.0*t*dt*dt + 0.65*3.0*t*t*dt+0.83*t*t*t;
}

vec2 getUV(vec2 uv, float fstep)
{
    vec4 current_pos_range = (pos_range - vec4(screenSize/2.0, screenSize/2.0)) / vec4(screenSize, screenSize);//计算出和屏幕的比例 vec4(start_x_f,start_y_f,end_x_f,end_y_f)
    vec2 center = vec2(0.5);//屏幕中间的比例点

    float t = (angle_range.x + (angle_range.y - angle_range.x) * fstep)  * PI / 180.;//角度转弧度

    float zoom = zoom_range.x + (zoom_range.y - zoom_range.x) * fstep;//fstep 这个时间点下的缩放的比例

    float sx = 1./zoom, sy = 1./zoom;

    vec2 pos_start = current_pos_range.xy * -1.0;
    vec2 pos_end = current_pos_range.zw * -1.0;

    float tx = pos_start.x + (pos_end.x - pos_start.x) * fstep;//tx:translationX ，fstep 这个时间点下的偏移量
    float ty = pos_start.y + (pos_end.y - pos_start.y) * fstep;
    vec2 trans = vec2(tx, ty);

    vec2 futureUv = (uv + trans - center);

    //t:弧度 sx,sy:缩放
    mat3 rot = mat3(sx * cos(t), -sx/1.78 * sin(t), 0.,
    sy * sin(t)*1.78, sy * cos(t), 0.,
    0., 0., 1.0);
    vec3 ruv = rot * vec3(futureUv, 1.0);
    futureUv = center + ruv.xy;//然后再放回去

    return futureUv;
}

void main() {
    vec2 uv = vTextureCo;//texture 原来的 uv

    vec2 splitUv = vec2(uv.xy);

    float time_length = (time_range.y - time_range.x);
    float fstep0 = (uTime-time_range.x) / time_length;//位于整个动画的什么位置
    if (fstep0 > 1.0) fstep0 = 1.0;
    float temp0 = (1.0-fstep0);
    if (interpolator_enable){
        fstep0 = bezierInterpolate(fstep0);
    } else if (fstep0 > 1.0){
        fstep0 = 1.0;
    }
    vec2 muv = getUV(splitUv, fstep0);

    if (motion_blur_enable){
        float fstep1 = (uTime-time_range.x + 10.) / time_length;//一段时间之后的影像
        if (fstep1 > 1.0) fstep1 = 1.0;
        if (interpolator_enable){
            fstep1 = bezierInterpolate(fstep1);
        } else if (fstep1 > 1.0){
            fstep1 = 1.0;
        }

        vec2 tuv = getUV(splitUv, fstep1);

        float step = 25.0;
        int istep = int(step);
        
        float count = 0.0;
        vec4 result = vec4(0.0);

        vec2 dir_step = (tuv.xy - muv) / step;
        for (int i = 0; i < istep; i++) {
            vec2 xuv = muv + dir_step * float(i);
            if ((xuv.x >=0.0) && (xuv.y >= 0.0) && (xuv.x <= 1.0) && (xuv.y <= 1.0)) {
                result += texture2D(vTexture, xuv);
                count += 1.0;
            }

            if (mirror_fill){
                if (xuv.x < 0.0){
                    result += texture2D(vTexture, vec2(-xuv.x, xuv.y));
                    count += 1.0;
                }

                if (xuv.x > 1.0){
                    result += texture2D(vTexture, vec2(2.0 - xuv.x, xuv.y));
                    count += 1.0;
                }

                if (xuv.y > 1.0){
                    result += texture2D(vTexture, vec2(xuv.x, 2.0 - xuv.y));
                    count += 1.0;
                }

                if (xuv.y < 0.0){
                    result += texture2D(vTexture, vec2(xuv.x, -xuv.y));
                    count += 1.0;
                }
            }

            if (compact_enable){
                if (xuv.x < 0.0){
                    gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
                }

                if (xuv.x > 1.0){
                    gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
                }

                if (xuv.y > 1.0){
                    gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
                }

                if (xuv.y < 0.0){
                    gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
                }
            }
        }

        gl_FragColor = vec4(result.xyz / step, 1.0);// 除以 step 取回平均值
    } else {
        gl_FragColor = texture2D(vTexture, muv);
    }
}